#!/bin/bash

initEnv () {
    SERVICE_BIN_FOLDER="$(cd $(dirname "${BASH_SOURCE[0]}") && pwd)"

    BINARY="${JF_PRODUCT_HOME}/app/${SERVICE_NAME}/bin/${BIN_NAME}"
    DOCKER_ENV_FILE="/.dockerenv"
    K8S_ENV_FOLDER="/var/run/secrets/kubernetes.io"

    if [ -f ${DOCKER_ENV_FILE} ] && [ -d ${K8S_ENV_FOLDER} ]; then
        echo "Running on k8s environment"
        export SERVICE_K8S_ENV=${SERVICE_K8S_ENV:-'true'}
    fi
}

sourceScript(){
    local file=$1

    [ ! -z "${file}" ] || errorExit "target file is not passed to source a file"

    if [ ! -f "${file}" ]; then
        errorExit "${file} file is not found"
    else
        source "${file}" || errorExit "Unable to source ${file}, please check if the $USER user has permissions to perform this action"
    fi
}

initHelpers(){
    #local systemYamlHelper="${SERVICE_BIN_FOLDER}"/systemYamlHelper.sh
    #local installerCommon="${SERVICE_BIN_FOLDER}"/installerCommon.sh
    export YQ_PATH="${SERVICE_BIN_FOLDER}/../third-party/yq"
    #sourceScript "${installerCommon}"
    #sourceScript "${systemYamlHelper}"
    helpers=(
    "_constants.sh"
    "_logger.sh"
    "_processHelper.sh"
    "_ioOperations.sh"
    "_validations.sh"
    "_envOperations.sh"
    "_dockerOperations.sh"
    "_wrapper_functions.sh"
    "_systemYamlHelper.sh"
    "_standaloneInstaller.sh"
)
    for helper in "${helpers[@]}"; do
        [ -f "${SERVICE_BIN_FOLDER}/helpers/$helper" ] || { echo "${SERVICE_BIN_FOLDER}/helpers/$helper does not exist."; exit 1; }
        source "${SERVICE_BIN_FOLDER}/helpers/$helper"
    done

    PRODUCT_NAME=$SERVICE_NAME
    export JF_SYSTEM_YAML="${JF_PRODUCT_HOME}/var/etc/system.yaml"
    setupScriptLogsRedirection || true
}

validateBinary(){
    [ -f "${BINARY}" ] || errorExit "Binary not found at ${BINARY}"
    [ -x "${BINARY}" ] || chmod +x "${BINARY}" || errorExit "Failed to make binary executable : ${BINARY}"
}

initEnv
initHelpers
commonStartupActions "docker" 2>/dev/null
testVarPermissions
validateBinary
displayEnv

logger "${SERVICE_NAME} service container is starting..."
logger "Exec: ${BINARY}"

logDir="${JF_PRODUCT_HOME}/var/log"
crashLogFile="${logDir}/${SERVICE_NAME}-error.log"

# Check if the crashLogFile file exists, if not, create it
if [ ! -f "${crashLogFile}" ]; then
  mkdir -p "${logDir}"
  touch "${crashLogFile}"
fi

internalServiceName() {
  declare -A serviceMap
  serviceMap=(
      ["catalog"]="jfcatalog"
  )

  result=${serviceMap[$SERVICE_NAME]}
  if [ -z "$result" ]; then
      echo "${SERVICE_NAME}"
  else
      echo "${result}"
  fi
}

crashLogToConsole() {
    # echoes a the formatted log to stdout for it to be scrapped by coralogix
    local logName="${SERVICE_NAME}-error-service.log"
    local dateTime=$(date -u +"%Y-%m-%dT%H:%M:%S.%3NZ")
    local serviceName=$(internalServiceName)

    # take the first 30000 characters (since coralogix can parse up to 32k, and escape new lines to get a valid JSON
    local logData=$(head -c 30000 "${crashLogFile}" | grep -v "maxprocs:" | awk '{printf "%s\\n", $0}' | sed -e 's/\t/\\t/g' )
    printf '{"log_name": "%s", "app": {"loglevel": "panic", "class": "wrapper", "datetime": "%s","service": "%s","message": "%s"}}\n' "$logName" "$dateTime" "$serviceName" "$logData"
}

postprocessCrashDump() {
    # delete dumps older than 7 days
    find ${logDir}/${SERVICE_NAME}-error.*.log -type f -mtime +7 -delete;

    if [ ! -f "${crashLogFile}" ]; then
        return
    fi

    # the file will allways have one line since automaxprocs echoes to stderr, so we are checking if it contains more than one line
    # Panics on internal errors as well. don't publish the log if it's the case, since those logs are already published
    if [ $(wc -l < "${crashLogFile}") -gt 1 ] && ! grep -q "Exiting from Service" "${crashLogFile}"; then
        crashLogToConsole
    fi
    mkdir -p "${logDir}/errors"
    local newLogFilePath="${logDir}/errors/${SERVICE_NAME}-error-service.$(date +"%s").log"

    mv "${crashLogFile}" "${newLogFilePath}"
}

terminate () {
    logger "Terminate signal received, attempting a graceful shutdown of Service-${SERVICE_NAME}"
    kill ${servicePid}
    if isProcessRunning "${servicePid}" "${BIN_NAME}" "${SERVICE_SHUTDOWN_WAIT_TIME}" 2>/dev/null; then
      warn "Could not perform a graceful shutdown, trying a hard kill"
      kill -9 ${servicePid}
    fi
    # cleanup any potential crash dumps
    postprocessCrashDump
    if [ "$SERVICE_K8S_ENV" = "true" ]; then
      logger "Waiting 10 seconds before termination"
      sleep 10
    fi
}

PID_FILE="${JF_PRODUCT_HOME}/var/${SERVICE_NAME}.pid"
SERVICE_SHUTDOWN_WAIT_TIME=20

trap terminate SIGINT SIGTERM SIGHUP

if $(isConsoleLogDisabled >/dev/null 2>&1); then
    # if console log is disabled, write stderr to crashLogFile, and back to stdout
    exec ${BINARY} 2> >(tee "${crashLogFile}" ) &
else
    # else, write stderr to a dedicated crashLogFile, and then to stdout, so it will be handled by
    # redirectServiceLogsToFile as well
    exec ${BINARY} 2> >(tee "${crashLogFile}" ) > >(tee >( redirectServiceLogsToFile ) ) &
fi
servicePid=$!

if [ -n "${PID_FILE}" ];
then
    mkdir -p $(dirname "${PID_FILE}") || \
    errorExit "Could not create parent dir for ${PID_FILE}";
fi

echo "${servicePid}" > ${PID_FILE}

wait ${servicePid}

postprocessCrashDump

logger "Waiting 10 seconds after service shutdown"
[ "$SERVICE_K8S_ENV" = "true" ] && sleep 10